"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
// import css
require("../../css/sheet-tab.css");
function SheetTabActions(props) {
    const onDelete = () => {
        var _a;
        (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
            userId: window.user_id,
            event: 'dataframe_delete_log_event',
            properties: {
                sheet_index: props.sheetIndex
            }
        });
        props.send({
            'event': 'edit_event',
            'type': 'dataframe_delete_edit',
            'sheet_index': props.sheetIndex,
        });
        props.setDisplayActions(false);
    };
    const onDuplicate = () => {
        var _a;
        (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
            userId: window.user_id,
            event: 'dataframe_duplicate_log_event',
            properties: {
                sheet_index: props.sheetIndex
            }
        });
        props.send({
            'event': 'edit_event',
            'type': 'dataframe_duplicate_edit',
            'sheet_index': props.sheetIndex,
        });
        props.setDisplayActions(false);
    };
    const onRename = () => {
        props.setDisplayActions(false);
        props.setIsRename(true);
    };
    return (react_1.default.createElement("div", { className: 'sheet-tab-actions-dropdown', style: { left: props.getRelativeLocation() + 1 } },
        react_1.default.createElement("div", { className: 'sheet-tab-action', onClick: onDelete }, "Delete"),
        react_1.default.createElement("div", { className: 'sheet-tab-action', onClick: onDuplicate }, "Duplicate"),
        react_1.default.createElement("div", { className: 'sheet-tab-action', onClick: onRename }, "Rename")));
}
function SheetTab(props) {
    const [displayActions, setDisplayActions] = react_1.useState(false);
    const [isRename, setIsRename] = react_1.useState(false);
    const [newDataframeName, setNewDataframeName] = react_1.useState(props.dfName);
    const selectedClass = props.selectedSheetIndex == props.sheetIndex ? ' selected-tab' : '';
    const elRef = react_1.useRef(null);
    function getRelativeLocation() {
        var _a, _b, _c, _d;
        const containerRectLeft = (_b = (_a = elRef.current) === null || _a === void 0 ? void 0 : _a.getBoundingClientRect()) === null || _b === void 0 ? void 0 : _b.left;
        const tabRectLeft = (_d = (_c = props.mitoContainerRef) === null || _c === void 0 ? void 0 : _c.getBoundingClientRect()) === null || _d === void 0 ? void 0 : _d.left;
        if (containerRectLeft && tabRectLeft) {
            return containerRectLeft - tabRectLeft;
        }
        return 0;
    }
    const onRename = () => {
        var _a;
        (_a = window.logger) === null || _a === void 0 ? void 0 : _a.track({
            userId: window.user_id,
            event: 'dataframe_rename_log_event',
            properties: {
                sheet_index: props.sheetIndex,
                new_dataframe_name: newDataframeName
            }
        });
        props.send({
            'event': 'edit_event',
            'type': 'dataframe_rename_edit',
            'sheet_index': props.sheetIndex,
            'new_dataframe_name': newDataframeName
        });
        setDisplayActions(false);
        setIsRename(false);
    };
    return (react_1.default.createElement("div", { ref: elRef, className: 'tab' + selectedClass, onClick: () => { props.setSelectedSheetIndex(props.sheetIndex); } },
        react_1.default.createElement("div", null,
            react_1.default.createElement("div", { className: 'tab-text' },
                isRename &&
                    react_1.default.createElement("form", { onSubmit: (e) => { console.log("test"); e.preventDefault(); onRename(); } },
                        react_1.default.createElement("input", { type: 'text', value: newDataframeName, onChange: (e) => { setNewDataframeName(e.target.value); }, autoFocus: true })),
                !isRename &&
                    react_1.default.createElement("p", { className: 'tab-sheet-name' }, props.dfName),
                react_1.default.createElement("svg", { onClick: () => { setDisplayActions(!displayActions); getRelativeLocation(); }, width: "20", height: "20", viewBox: "0 0 9 4", fill: "none", xmlns: "http://www.w3.org/2000/svg" },
                    react_1.default.createElement("path", { d: "M4.50098 4L8.39809 0.25H0.603862L4.50098 4Z", fill: "#C4C4C4" })))),
        displayActions &&
            react_1.default.createElement(SheetTabActions, { setDisplayActions: setDisplayActions, setIsRename: setIsRename, send: props.send, dfName: props.dfName, sheetIndex: props.sheetIndex, getRelativeLocation: getRelativeLocation })));
}
exports.default = SheetTab;
//# sourceMappingURL=SheetTab.js.map